@@ -386,7 +386,8 @@ is_appointment (GtkTreeModel *model,
 	gtk_tree_model_get (model, iter, APPOINTMENT_COLUMN_URI, &uri, -1);
 	if (uri)
 		return (strcmp (uri, "file") == 0 ||
-			strcmp (uri, "webcal") == 0);
+			strcmp (uri, "webcal") == 0 ||
+			strcmp (uri, "caldav") == 0);
 	return FALSE;
 }
 
@@ -772,6 +773,9 @@ create_task_list (CalendarWindow *calwin,
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
                                         GTK_POLICY_NEVER,
                                         GTK_POLICY_AUTOMATIC);
+	/* we show the widget before adding to the container, since adding to
+	 * the container changes the visibility depending on the state of the
+	 * expander */
         gtk_widget_show (scrolled);
         gtk_container_add (GTK_CONTAINER (list), scrolled);
 
@@ -943,8 +947,11 @@ create_list_for_appointment_model (CalendarWindow      *calwin,
         gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled),
                                         GTK_POLICY_NEVER,
                                         GTK_POLICY_AUTOMATIC);
-        gtk_container_add (GTK_CONTAINER (list), scrolled);
+	/* we show the widget before adding to the container, since adding to
+	 * the container changes the visibility depending on the state of the
+	 * expander */
         gtk_widget_show (scrolled);
+        gtk_container_add (GTK_CONTAINER (list), scrolled);
 
 	g_assert (calwin->priv->appointments_model != NULL);
 
@@ -1151,12 +1158,25 @@ calendar_day_activated (GtkCalendar    *calendar,
 	unsigned int  day;
 	unsigned int  month;
 	unsigned int  year;
+	time_t        now;
+	struct tm     utc_tm;
+	struct tm     local_tm;
 	char         *argument;
 
 	gtk_calendar_get_date (calendar, &year, &month, &day);
 
-	argument = g_strdup_printf ("calendar:///?startdate=%.4d%.2d%.2d",
-				    year, month + 1, day);
+	time (&now);
+	gmtime_r (&now, &utc_tm);
+	localtime_r (&now, &local_tm);
+
+	/* FIXME: once bug 409200 is fixed, we'll have to make this hh:mm:ss
+	 * instead of hhmmss */
+	argument = g_strdup_printf ("calendar:///?startdate="
+				    "%.4d%.2d%.2dT%.2d%.2d%.2dZ",
+				    year, month + 1, day,
+				    12 + utc_tm.tm_hour - local_tm.tm_hour,
+				    0  + utc_tm.tm_min  - local_tm.tm_min,
+				    0);
 
 	clock_launch_evolution (calwin, argument);
 
@@ -1472,10 +1492,7 @@ static void add_child (GtkContainer *container,
                        GtkWidget    *child,
                        GtkExpander  *expander)
 {
-        gboolean expanded;
-
-        expanded = gtk_expander_get_expanded (expander);
-        g_object_set (child, "visible", expanded, NULL);
+	expand_collapse_child (child, expander);
 }
 
 static GtkWidget *
@@ -1555,6 +1572,9 @@ calendar_window_pack_locations (CalendarWindow *calwin, GtkWidget *vbox)
 							 KEY_LOCATIONS_EXPANDED,
 							 G_CALLBACK (edit_locations));
 
+	/* we show the widget before adding to the container, since adding to
+	 * the container changes the visibility depending on the state of the
+	 * expander */
 	gtk_widget_show (calwin->priv->locations_list);
 	gtk_container_add (GTK_CONTAINER (vbox), calwin->priv->locations_list);
 
