@@ -39,17 +39,6 @@
 #include "window-list.h"
 #include "showdesktop.h"
 
-void
-wncklet_set_tooltip (GtkWidget  *widget,
-		     const char *tip)
-{
-	char *markup;
-
-	markup = g_markup_escape_text (tip, -1);
-	g_object_set (widget, "tooltip-markup", markup, NULL);
-	g_free (markup);
-}
-
 void
 wncklet_display_about (GtkWidget   *applet,
 		       GtkWidget  **dialog,
@@ -72,7 +61,7 @@ wncklet_display_about (GtkWidget   *applet,
 
 	*dialog = gtk_about_dialog_new ();
 	g_object_set (*dialog,
-		      "name",  name,
+		      "program-name",  name,
 		      "version", VERSION,
 		      "copyright", copyright,
 		      "comments", comments,
