@@ -6,6 +6,7 @@
 #define PANEL_ICON_COMPUTER		"computer"
 #define PANEL_ICON_DESKTOP		"user-desktop"
 #define PANEL_ICON_DRAWER		"gnome-panel-drawer"
+#define PANEL_ICON_FILESYSTEM		"drive-harddisk"
 #define PANEL_ICON_FOLDER		"folder"
 #define PANEL_ICON_FONTS		"preferences-desktop-font"
 #define PANEL_ICON_FORCE_QUIT		"gnome-panel-force-quit"
@@ -22,6 +23,7 @@
 #define PANEL_ICON_REMOVABLE_MEDIA	"drive-removable-media"
 #define PANEL_ICON_RUN			"system-run"
 #define PANEL_ICON_SEPARATOR		"gnome-panel-separator"
+#define PANEL_ICON_SAVED_SEARCH		"folder-saved-search"
 #define PANEL_ICON_SEARCHTOOL		"system-search"
 #define PANEL_ICON_SHUTDOWN		"gnome-shutdown"
 #define PANEL_ICON_THEME		"preferences-desktop-theme"
